/* --------------------------------------------------------------------------
$Id: display_hlhdf.c,v 1.1.1.1 2001/12/14 12:02:39 ahenja Exp $
Program:        

Description:    Used for displaying a complete hlhdf NameList instance.

Author(s):      Anders Henja

Copyright:	Swedish Meteorological and Hydrological Institute, 2000

History:
$Log: display_hlhdf.c,v $
Revision 1.1.1.1  2001/12/14 12:02:39  ahenja
First CVS version of hlhdf

Revision 1.3  2000/03/22 08:28:29  ahenja
Updated with header.

-----------------------------------------------------------------------------*/

#include "display_hlhdf.h"

void displayDataBuffer(unsigned char* data, const char* fmt, int ndims,
		       hsize_t* dims, size_t typeSize,int offs,int addNewline)
{
   int j,x,y;
   char buff[1024];
   hsize_t dimsctr[2];
   int xsize;
   size_t didx;

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(ndims>2) {
      printf("%sDATA IS OF GREATER DIMENSION THAN 2, NOT SUPPORTED\n",buff);
      return;
   }

   dimsctr[0]=1;
   dimsctr[1]=1;

   if(ndims==1) {
      dimsctr[1]=dims[0];
   } else if(ndims==2) {
      dimsctr[0]=dims[0];
      dimsctr[1]=dims[1];
   }
   xsize=dimsctr[1];

   for(y=0;y<dimsctr[0];y++) {
      printf("%s",buff);
      for(x=0;x<dimsctr[1];x++) {
	 didx=(y*xsize+x)*typeSize;
	 if(strcmp(fmt,"char")==0) {
	    char mc;
	    memcpy(&mc,&data[didx],typeSize);
	    printf("%c",mc);
	 } else if(strcmp(fmt,"schar")==0) {
	    signed char mc;
	    memcpy(&mc,&data[didx],typeSize);
	    printf("%d",mc);
	 } else if(strcmp(fmt,"uchar")==0) {
	    unsigned char mc;
	    memcpy(&mc,&data[didx],typeSize);
	    printf("%u",mc);
	 } else if(strcmp(fmt,"short")==0) {
	    short ms;
	    memcpy(&ms,&data[didx],typeSize);
	    printf("%d",ms);
	 } else if(strcmp(fmt,"ushort")==0) {
	    unsigned short ms;
	    memcpy(&ms,&data[didx],typeSize);
	    printf("%u",ms);
	 } else if(strcmp(fmt,"int")==0) {
	    int mi;
	    memcpy(&mi,&data[didx],typeSize);
	    printf("%d",mi);
	 } else if(strcmp(fmt,"uint")==0) {
	    unsigned int mi;
	    memcpy(&mi,&data[didx],typeSize);
	    printf("%d",mi);
	 } else if(strcmp(fmt,"long")==0) {
	    long ml;
	    memcpy(&ml,&data[didx],typeSize);
	    printf("%ld",ml);
	 } else if(strcmp(fmt,"ulong")==0) {
	    unsigned long ml;
	    memcpy(&ml,&data[didx],typeSize);
	    printf("%lu",ml);
	 } else if(strcmp(fmt,"llong")==0) {
	    long long ml;
	    memcpy(&ml,&data[didx],typeSize);
	    printf("%ld",(long)ml);
	 } else if(strcmp(fmt,"ullong")==0) {
	    unsigned long long ml;
	    memcpy(&ml,&data[didx],typeSize);
	    printf("%lu",(unsigned long)ml);
	 } else if(strcmp(fmt,"float")==0) {
	    float mf;
	    memcpy(&mf,&data[didx],typeSize);
	    printf("%g",mf);
	 } else if(strcmp(fmt,"double")==0) {
	    double md;
	    memcpy(&md,&data[didx],typeSize);
	    printf("%g",md);
	 } else if(strcmp(fmt,"ldouble")==0) {
	    double md;
	    memcpy(&md,&data[didx],typeSize);
	    printf("%lg",md);
	 } else if(strcmp(fmt,"hssize")==0) {
	    hssize_t mh;
	    memcpy(&mh,&data[didx],typeSize);
	    if(sizeof(hssize_t)==sizeof(int)) {
	       printf("%d",(int)mh);
	    } else if(sizeof(hssize_t)==sizeof(long)) {
	       printf("%ld",(long)mh);
	    } else {
	       printf("%ld",(long)mh);
	    }
	 } else if(strcmp(fmt,"hsize")==0) {
	    hsize_t mh;
	    memcpy(&mh,&data[didx],typeSize);
	    if(sizeof(hsize_t)==sizeof(int)) {
	       printf("%u",(unsigned int)mh);
	    } else if(sizeof(hsize_t)==sizeof(long)) {
	       printf("%lu",(unsigned long)mh);
	    } else {
	       printf("%lu",(unsigned long)mh);
	    }
	 } else if(strcmp(fmt,"string")==0) {
	    char* ms=malloc(typeSize+1);
	    if(!ms) {
	       printf("\"\"");
	    } else {
	       memcpy(ms,&data[didx],typeSize);
	       ms[typeSize]='\0';
	       printf("\"%s\"",ms);
	       free(ms);
	    }
	    continue;
	 } else {
	    if(_debug_hdf)
	       fprintf(stderr,"Unknown data format, cannot print\n");
	    return;
	 }
	 if(x<dimsctr[1]-1)
	    printf(", ");
      }
      if(addNewline || j<dimsctr[0]-1)
	 printf("\n");
   }   
}

#ifdef KALLE
void displayDataBuffer(unsigned char* data, const char* fmt, int ndims,
		       hsize_t* dims, size_t typeSize,int offs,int addNewline)
{
   int j,x,y;
   char buff[1024];
   hsize_t dimsctr[2];
   int xsize;

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(ndims>2) {
      printf("%sDATA IS OF GREATER DIMENSION THAN 2, NOT SUPPORTED\n",buff);
      return;
   }

   dimsctr[0]=1;
   dimsctr[1]=1;

   if(ndims==1) {
      dimsctr[1]=dims[0];
   } else if(ndims==2) {
      dimsctr[0]=dims[0];
      dimsctr[1]=dims[1];
   }
   xsize=dimsctr[1];

   printf("Before for-loop, fmt=%s,typeSize=%d\n",fmt,typeSize);

   for(y=0;y<dimsctr[0];y++) {
      printf("%s",buff);
      for(x=0;x<dimsctr[1];x++) {
	 if(strcmp(fmt,"char")==0) {
	    printf("%c",((char*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"schar")==0) {
	    printf("%d",((signed char*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"uchar")==0) {
	    printf("%u",((unsigned char*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"short")==0) {
	    printf("%d",((short*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"ushort")==0) {
	    printf("%u",((unsigned short*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"int")==0) {
	    printf("%d",((int*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"uint")==0) {
	    printf("%d",((unsigned int*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"long")==0) {
	    printf("%ld",((long*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"ulong")==0) {
	    printf("%lu",((unsigned long*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"llong")==0) {
	    printf("%ld",((long long*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"ullong")==0) {
	    printf("%lu",((unsigned long long*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"float")==0) {
	    printf("%g",((float*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"double")==0) {
	    printf("%g",((double*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"ldouble")==0) {
	    printf("%lg",((double*)data)[y*xsize+x]);
	 } else if(strcmp(fmt,"hssize")==0) {
	    if(sizeof(hssize_t)==sizeof(int)) {
	       printf("%d",((int*)data)[y*xsize+x]);
	    } else if(sizeof(hssize_t)==sizeof(long)) {
	       printf("%ld",((long*)data)[y*xsize+x]);
	    } else {
	       printf("%ld",((long long*)data)[y*xsize+x]);
	    }
	 } else if(strcmp(fmt,"hsize")==0) {
	    if(sizeof(hsize_t)==sizeof(int)) {
	       printf("%u",((unsigned*)data)[y*xsize+x]);
	    } else if(sizeof(hsize_t)==sizeof(long)) {
	       printf("%lu",((unsigned long*)data)[y*xsize+x]);
	    } else {
	       printf("%lu",((unsigned long long*)data)[y*xsize+x]);
	    }
	 } else if(strcmp(fmt,"string")==0) {
	    char* mystr=&((char*)data)[y*xsize];
	    printf("\"%s\"",mystr);
	    continue;
	 } else {
	    if(_debug_hdf)
	       fprintf(stderr,"Unknown data format, cannot print\n");
	    return;
	 }
	 if(x<dimsctr[1]-1)
	    printf(", ");
      }
      if(addNewline || j<dimsctr[0]-1)
	 printf("\n");
   }   

   printf("After for-loop\n");

}
#endif

void displayCompoundDataset(unsigned char* data,NameListType_t* type,int ndims, hsize_t* dims, int offs)
{
   int i,j,x,y;
   char buff[1024];
   hsize_t dimsctr[2];
   int xsize;
   unsigned char* aDptr=data;
   unsigned long compoundIndex=0;
   unsigned long attrIndex=0;
   size_t datasize=type->typeSize;
   int nctr;
   int howmany;
   int iii;

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(ndims>2) {
      printf("%sDATA IS OF GREATER DIMENSION THAN 2, NOT SUPPORTED\n",buff);
      return;
   }

   dimsctr[0]=1;
   dimsctr[1]=1;

   if(ndims==1) {
      dimsctr[1]=dims[0];
   } else if(ndims==2) {
      dimsctr[0]=dims[0];
      dimsctr[1]=dims[1];
   }
   xsize=dimsctr[1];

   for(y=0;y<dimsctr[0];y++) {
      for(x=0;x<dimsctr[1];x++) {
	 printf("%s{\n",buff);
	 attrIndex = 0;
	 for(j=0;j<type->noAttrs;j++) {
	    aDptr = &data[compoundIndex+type->attrs[j]->offset];
	    printf("%s  [ ",buff);
	    displayDataBuffer(aDptr,type->attrs[j]->format_type,type->attrs[j]->ndims, \
			      type->attrs[j]->dims,type->attrs[j]->attrSize, 0 , 0);
	    printf(" ]\n");

	 }
	 printf("%s}\n",buff);

	 compoundIndex += datasize;
      }
   }
}

void displayCompoundAttributeDef(CompoundAttributeDef_t* def,int offs)
{
   int i,j;
   char buff[1024];

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(def->isString) {
      printf("%s{   STRSIZE %d\n",buff,def->attrSize);
      printf("%s    STRPAD  %s\n",buff,def->strpad);
      printf("%s    CSET    %s\n",buff,def->strcset);
      printf("%s    CTYPE   %s\n",buff,def->strctype);
      printf("%s    } \"%s\"",buff,def->name);
      if(def->ndims>0) {
	 for(i=0;i<def->ndims;i++) {
	    printf("[%d]",(int)def->dims[i]);
	 }
      }
      printf(";\n");
   } else {
      printf("%s%s \"%s\"",buff,def->name,def->ieee_type);
      if(def->ndims>0) {
	 for(i=0;i<def->ndims;i++) {
	    printf("[%d]",(int)def->dims[i]);
	 }
      }
      printf(";\n");
   }
}

void displayType(NameListType_t* type, int offs)
{
   int i,j;
   char buff[1024];

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(type->isNamed || type->name)
      printf("%sDATATYPE \"%s\" {\n",buff,type->name);
   else
      printf("%sDATATYPE {\n",buff);

   for(i=0;i<type->noAttrs;i++) {
      displayCompoundAttributeDef(type->attrs[i],offs+DISPLAY_INDENT);
   }

   printf("%s}\n",buff); /*For datatype block*/
}

void displayAttribute(NameListAttribute_t* attr,int offs)
{
   int i,j;
   char buff[1024];
   NameListType_t* nltype=NULL;

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(!attr)
      printf("No attribute, something very strange\n");

   if(!attr->name)
      printf("No attribute name ???\n");

   nltype=attr->attrtype;

   if(!nltype) {
      printf("%sNo namelist type connected to attribute\n",buff);
   }

   printf("%sATTRIBUTE \"%s\" {\n",buff,attr->name);

   if(!nltype->isCompound) {
      if(nltype->ieee_type && strcmp(nltype->ieee_type,"H5T_STRING")==0) {
	 printf("%s  DATATYPE {\n",buff);
	 printf("%s    { STRSIZE %d\n",buff,nltype->typeSize);
	 printf("%s      STRPAD  %s\n",buff,nltype->strpad);
	 printf("%s      CSET    %s\n",buff,nltype->strcset);
	 printf("%s      CTYPE   %s\n",buff,nltype->strctype);
	 printf("%s    }\n",buff);
      } else {
	 printf("%s  DATATYPE { %s }\n",buff,nltype->ieee_type);
      }
   } else {
      if(nltype->name) {
	 printf("%s  DATATYPE {\n",buff);
	 printf("%s    \"%s\"\n",buff,nltype->name);
	 printf("%s  }\n",buff);
      } else {
	 displayType(nltype,offs+DISPLAY_INDENT);
      }
   }
   if(attr->ndims) {
      printf("%s  DATASPACE { SIMPLE (",buff);
      for(j=0;j<attr->ndims;j++) {
	 printf("%d",(int)attr->dims[j]);
	 if(j<attr->ndims-1)
	    printf(",");
      }
      printf(") }\n");
   } else {
      printf("%s  DATASPACE { SCALAR }\n",buff);
   }

   printf("%s  DATA {\n",buff);

   if(!nltype->isCompound) {
      displayDataBuffer(attr->data,nltype->format_type,attr->ndims,attr->dims,nltype->typeSize,offs+DISPLAY_INDENT+2,1);
   } else {
      displayCompoundDataset(attr->data,nltype,attr->ndims,attr->dims,offs+DISPLAY_INDENT+2);
   }

   printf("%s  }\n",buff); /*For data block*/

   printf("%s}\n",buff); /*For attribute block*/
}

void displayDataset(NameListDataset_t* dset, int offs)
{
   int i,j;
   char buff[1024];
   NameListType_t* nltype=NULL;

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';

   if(!dset)
      printf("No dataset, something very strange\n");
   if(!dset->name)
      printf("Dataset does not have any name, very strange\n");

   nltype=dset->datatype;

   if(!nltype)
      printf("Dataset does not have any type connected to it\n");

   if(!nltype->isCompound) {
      printf("%sDATASET \"%s\" {\n",buff,dset->name);
      if(nltype->ieee_type && strcmp(nltype->ieee_type,"H5T_STRING")==0) {
	 printf("%s  DATATYPE {\n",buff);
	 printf("%s    { STRSIZE %d\n",buff,nltype->typeSize);
	 printf("%s      STRPAD  %s\n",buff,nltype->strpad);
	 printf("%s      CSET    %s\n",buff,nltype->strcset);
	 printf("%s      CTYPE   %s\n",buff,nltype->strctype);
	 printf("%s    }\n",buff);
      } else {
	 printf("%s  DATATYPE { %s }\n",buff,nltype->ieee_type);
      }
   } else {
      printf("%sDATASET \"%s\" {\n",buff,dset->name);
      if(nltype->name) {
	 printf("%s  DATATYPE {\n",buff);
	 printf("%s    \"%s\"\n",buff,nltype->name);
	 printf("%s  }\n",buff);
      } else {
	 displayType(nltype,offs+DISPLAY_INDENT);
      }
   }

   if(dset->ndims) {
      printf("%s  DATASPACE { SIMPLE (",buff);
      for(j=0;j<dset->ndims;j++) {
	 printf("%d",(int)dset->dims[j]);
	 if(j<dset->ndims-1)
	    printf(",");
      }
      printf(") }\n");
   } else {
      printf("%s  DATASPACE { SCALAR }\n",buff);
   }

   printf("%s  DATA {\n",buff);

   if(!nltype->isCompound) {
      displayDataBuffer(dset->data,nltype->format_type, \
			dset->ndims,dset->dims,nltype->typeSize,offs+DISPLAY_INDENT+2,1);
   } else {
      displayCompoundDataset(dset->data,nltype,dset->ndims,dset->dims,offs+DISPLAY_INDENT+2);
   }

   printf("%s  }\n",buff); /*For data block*/

   for(i=0;i<dset->noAttrs;i++) {
      displayAttribute(dset->attrs[i],offs+DISPLAY_INDENT);
   }   

   printf("%s}\n",buff); /*For dataset block*/   

}


void displayGroup(NameListGroup_t* grp,int offs)
{
   int i,j;
   char buff[1024];

   for(j=0;j<offs;j++)
      buff[j]=' ';
   buff[offs]='\0';
   printf("%sGROUP \"%s\" {\n",buff,grp->name);

   for(i=0;i<grp->noLocalTypes;i++) {
      displayType(grp->localTypes[i],offs+DISPLAY_INDENT);
   }

   for(i=0;i<grp->noGroups;i++) {
      displayGroup(&grp->groups[i],offs+DISPLAY_INDENT);
   }

   for(i=0;i<grp->noAttrs;i++) {
      displayAttribute(grp->attrs[i],offs+DISPLAY_INDENT);
   }

   for(i=0;i<grp->noDatasets;i++) {
      displayDataset(&grp->datasets[i],offs+DISPLAY_INDENT);
   }

   printf("%s}\n",buff); /*For the GROUP*/
}
